"""
Middleware functions test
"""
import unittest
import unittest.mock as mock
import xmlrunner
from nzsp_pms_utils.middleware import verify_authorization_decorator

class TestMiddleware(unittest.TestCase):
    """
    Test
    """

    @verify_authorization_decorator()
    def mock_route(self):
        """
        This method help us to test the decorator
        """
        pass

    @mock.patch('nzsp_pms_utils.middleware.flask')
    def test_verify_authorization_decorator_raises_key_error(self, mock_flask):
        """Raises KeyError when no service_name set globally test"""
        mock_flask.g.get.return_value = None
        with self.assertRaises(KeyError):
            self.mock_route()

    @mock.patch('nzsp_pms_utils.middleware.flask')
    @mock.patch('nzsp_pms_utils.middleware.requests')
    def test_verify_authorization_decorator_status_200(self, mock_requests, mock_flask):
        """Assign user identifier globally in status 200 test"""
        mock_flask.g.get.return_value = {
            "service_name": "Test"
        }
        user_identifier = "Test"
        mock_requests.get.return_value.status_code = 200
        mock_requests.get.return_value.json.return_value = {
            "user_identifier": user_identifier
        }
        self.mock_route()
        self.assertEqual(mock_flask.g.user_identifier, user_identifier)

    @mock.patch('nzsp_pms_utils.middleware.flask')
    @mock.patch('nzsp_pms_utils.middleware.requests')
    def test_verify_authorization_decorator_status_500(self, mock_requests, mock_flask):
        """Response jsonify status in status different that 200 test"""
        mock_flask.g.get.return_value = {
            "service_name": "Test"
        }
        status_code = 500
        mock_requests.get.return_value.status_code = 500
        self.mock_route()
        mock_flask.Response.assert_called_with(status = status_code)

if __name__ == '__main__':
    unittest.main(
        testRunner=xmlrunner.XMLTestRunner(output='test-reports'),
        # these make sure that some options that are not applicable
        # remain hidden from the help menu.
        failfast=False, buffer=False, catchbreak=False
    )



"""Contains NZSP PMS flask server middleware"""
import os
import requests
import flask
from flask.wrappers import Response
from functools import wraps

def verify_authorization_decorator() -> Response:
    """
    Verify authorization using Plant Managment system
    IAM Authorize API. if status 200 (OK), sets user identifier
    globally.

    Returns
    -------
    Response
        if status different than 200 (OK), reponse status.

    Raises
    ------
    KeyError
        When service_name is not set globally.

    Notes
    -----
    - Requires to set up the service name globally in
    an application before request.
    - Requires to set up environment vatiable IAM_AUTHORIZE_API.

    See Also
    --------
    - Check PMS IAM documentation to know more about this middleware.

    Examples
    --------
    Setting up application before request
    >>> @app.before_request
    ... def set_service_name_globally():
    ...    g.service_name = SERVICE_NAME
    Calling it in a route
    >>> from nzsp_pms_utils.middleware import verify_authorization_decorator
    ... @bp.route("/service", methods=["GET"])
    ... @verify_authorization_decorator()
    ... def on_get_user_services():
    ...    # your method here
    """
    def _verify_authorization_decorator(f):
        @wraps(f)
        def _verify_authorization_wrapper(*args, **kwargs):
            if not flask.g.get("service_name"):
                raise KeyError(
                    "service_name was not set globally," +
                    "read the docs to get more info"
                )
            iam_authorize_response = requests.get(
                url = os.getenv("IAM_AUTHORIZE_API"),
                headers = {
                    "authorization": flask.request.headers.get("authorization"),
                    "service_name": flask.g.get("service_name"),
                    "action_name": flask.request.method + flask.request.path
                }
            )
            if iam_authorize_response.status_code == 200:
                json_response = iam_authorize_response.json()
                flask.g.user_identifier = json_response["user_identifier"]
            else:
                return flask.Response(status = iam_authorize_response.status_code)
        return _verify_authorization_wrapper
    return _verify_authorization_decorator
